@@ -1,7 +1,7 @@
 use anyhow::Result;
 use std::io::Write;
 use std::str::FromStr;
-use tuc::bounds::{BoundsType, UserBoundsList};
+use tuc::bounds::{BoundOrFiller, BoundsType, UserBoundsList};
 use tuc::cut_bytes::read_and_cut_bytes;
 use tuc::cut_lines::read_and_cut_lines;
 use tuc::cut_str::read_and_cut_str;
@@ -33,6 +33,7 @@ FLAGS:
     -h, --help                    Print this help and exit
     -m, --complement              Invert fields (e.g. '2' becomes '1,3:')
     -j, --(no-)join               Print selected parts with delimiter in between
+    --json                        Print fields as a JSON array of strings
 
 OPTIONS:
     -f, --fields <bounds>         Fields to keep, 1-indexed, comma separated.
@@ -40,20 +41,20 @@ OPTIONS:
                                   Fields can be negative (-1 is the last field).
                                   [default 1:]
 
-                                  e.g. cutting on '-' the string 'a-b-c-d'
-                                  -f 1     => a
-                                  -f 1:    => a-b-c-d
-                                  -f 1:3   => a-b-c
-                                  -f 3,2   => cb
-                                  -f 3,1:2 => ca-b
-                                  -f -3:-2 => b-c
+                                  e.g. cutting the string 'a-b-c-d' on '-'
+                                    -f 1     => a
+                                    -f 1:    => a-b-c-d
+                                    -f 1:3   => a-b-c
+                                    -f 3,2   => cb
+                                    -f 3,1:2 => ca-b
+                                    -f -3:-2 => b-c
 
                                   To re-apply the delimiter add -j, to replace
                                   it add -r (followed by the new delimiter).
 
                                   You can also format the output using {} syntax
                                   e.g.
-                                  -f '["{1}", "{2}"]' => ["a", "b"]
+                                    -f '({1}, {2})' => (a, b)
 
                                   You can escape { and } using {{ and }}.
 
@@ -64,7 +65,8 @@ OPTIONS:
     -d, --delimiter <delimiter>   Delimiter used by --fields to cut the text
                                   [default: \t]
     -e, --regex <some regex>      Use a regular expression as delimiter
-    -r, --replace-delimiter <new> Replace the delimiter with the provided text
+    -r, --replace-delimiter <new> Replace the delimiter with the provided text.
+                                  Implies --join
     -t, --trim <type>             Trim the delimiter (greedy). Valid values are
                                   (l|L)eft, (r|R)ight, (b|B)oth
 
@@ -109,6 +111,13 @@ fn parse_args() -> Result<Opt, pico_args::Error> {
         BoundsType::Fields
     };
 
+    if bounds_type == BoundsType::Fields
+        && (maybe_fields.is_none() || maybe_fields.as_ref().unwrap().0.is_empty())
+    {
+        eprintln!("tuc: invariant error. At this point we expected to find at least 1 field bound");
+        std::process::exit(1);
+    }
+
     let delimiter = match bounds_type {
         BoundsType::Fields => pargs
             .opt_value_from_str(["-d", "--delimiter"])?
@@ -118,24 +127,57 @@ fn parse_args() -> Result<Opt, pico_args::Error> {
     };
 
     let greedy_delimiter = pargs.contains(["-g", "--greedy-delimiter"]);
-    let replace_delimiter = pargs.opt_value_from_str(["-r", "--replace-delimiter"])?;
+    let mut replace_delimiter = pargs.opt_value_from_str(["-r", "--replace-delimiter"])?;
 
+    let has_json = pargs.contains("--json");
     let has_join = pargs.contains(["-j", "--join"]);
     let has_no_join = pargs.contains("--no-join");
 
     if has_join && has_no_join {
-        eprintln!("tuc: runtime error. You can't pass both --join and --no-join");
+        eprintln!(
+            "tuc: runtime error. It's not possible to use --join and --no-join simultaneously"
+        );
         std::process::exit(1);
     }
 
-    if replace_delimiter.is_some() && has_no_join {
-        eprintln!("tuc: runtime error. Since --replace implies --join, you can't pass --no-join");
+    if has_json && has_no_join {
+        eprintln!("tuc: runtime error. Using both --json and --no-join is not permitted");
         std::process::exit(1);
     }
 
+    if replace_delimiter.is_some() {
+        if has_no_join {
+            eprintln!("tuc: runtime error. You can't pass --no-join when using --replace, which implies --join");
+            std::process::exit(1);
+        } else if has_json {
+            eprintln!("tuc: runtime error. The use of --replace with --json is not supported");
+            std::process::exit(1);
+        }
+    }
+
+    if bounds_type == BoundsType::Characters && has_no_join {
+        eprintln!(
+            "tuc: runtime error. Since --characters implies --join, you can't pass --no-join"
+        );
+        std::process::exit(1);
+    }
+
+    if has_json {
+        replace_delimiter = Some(",".to_owned());
+    }
+
     let join = has_join
+        || has_json
         || replace_delimiter.is_some()
-        || (bounds_type == BoundsType::Lines && !has_no_join);
+        || (bounds_type == BoundsType::Lines && !has_no_join)
+        || (bounds_type == BoundsType::Characters);
+
+    if has_json && bounds_type != BoundsType::Characters && bounds_type != BoundsType::Fields {
+        eprintln!(
+            "tuc: runtime error. --json support is available only for --fields and --characters"
+        );
+        std::process::exit(1);
+    }
 
     #[cfg(not(feature = "regex"))]
     let regex_bag = None;
@@ -154,8 +196,24 @@ fn parse_args() -> Result<Opt, pico_args::Error> {
             }),
         });
 
-    if regex_bag.is_some() && !cfg!(feature = "regex") {
-        eprintln!("tuc: runtime error. This version of tuc was compiled without regex support");
+    if regex_bag.is_some() && cfg!(not(feature = "regex")) {
+        eprintln!("tuc: invariant error. There should not be any regex when compiled without regex support");
+        std::process::exit(1);
+    }
+
+    let bounds = maybe_fields
+        .or(maybe_characters)
+        .or(maybe_bytes)
+        .or(maybe_lines)
+        .unwrap();
+
+    if has_json
+        && bounds
+            .0
+            .iter()
+            .any(|s| matches!(s, BoundOrFiller::Filler(_)))
+    {
+        eprintln!("tuc: runtime error. Cannot format fields when using --json");
         std::process::exit(1);
     }
 
@@ -171,13 +229,10 @@ fn parse_args() -> Result<Opt, pico_args::Error> {
             EOL::Newline
         },
         join,
+        json: has_json,
         delimiter,
         bounds_type,
-        bounds: maybe_fields
-            .or(maybe_characters)
-            .or(maybe_bytes)
-            .or(maybe_lines)
-            .unwrap(),
+        bounds,
         replace_delimiter,
         trim: pargs.opt_value_from_str(["-t", "--trim"])?,
         regex_bag,
