@@ -2,7 +2,10 @@ use anyhow::{bail, Result};
 use std::io::{BufRead, Write};
 use std::ops::Range;
 
-use crate::bounds::{bounds_to_std_range, BoundOrFiller, BoundsType};
+use crate::bounds::{
+    bounds_to_std_range, BoundOrFiller, BoundsType, Side, UserBounds, UserBoundsList,
+};
+use crate::json::escape_json;
 use crate::options::{Opt, Trim};
 use crate::read_utils::read_line_with_eol;
 
@@ -115,7 +118,9 @@ fn compress_delimiter_with_regex<'a>(
 
 #[cfg(feature = "regex")]
 fn maybe_replace_delimiter<'a>(text: &'a str, opt: &Opt) -> std::borrow::Cow<'a, str> {
-    if let Some(new_delimiter) = opt.replace_delimiter.as_ref() {
+    if opt.bounds_type == BoundsType::Characters {
+        std::borrow::Cow::Borrowed(text)
+    } else if let Some(new_delimiter) = opt.replace_delimiter.as_ref() {
         if let Some(re_bag) = &opt.regex_bag {
             re_bag.normal.replace_all(text, new_delimiter)
         } else {
@@ -128,7 +133,9 @@ fn maybe_replace_delimiter<'a>(text: &'a str, opt: &Opt) -> std::borrow::Cow<'a,
 
 #[cfg(not(feature = "regex"))]
 fn maybe_replace_delimiter<'a>(text: &'a str, opt: &Opt) -> std::borrow::Cow<'a, str> {
-    if let Some(new_delimiter) = opt.replace_delimiter.as_ref() {
+    if opt.bounds_type == BoundsType::Characters {
+        std::borrow::Cow::Borrowed(text)
+    } else if let Some(new_delimiter) = opt.replace_delimiter.as_ref() {
         std::borrow::Cow::Owned(text.replace(&opt.delimiter, new_delimiter))
     } else {
         std::borrow::Cow::Borrowed(text)
@@ -170,6 +177,18 @@ fn trim_regex<'a>(line: &'a str, trim_kind: &Trim, re: &Regex) -> &'a str {
     &line[idx_start..idx_end]
 }
 
+macro_rules! write_maybe_as_json {
+    ($writer:ident, $to_print:ident, $as_json:expr) => {{
+        if $as_json {
+            $writer.write_all(b"\"")?;
+            $writer.write_all(&escape_json(&$to_print).as_bytes())?;
+            $writer.write_all(b"\"")?;
+        } else {
+            $writer.write_all($to_print.as_bytes())?;
+        }
+    }};
+}
+
 pub fn cut_str<W: Write>(
     line: &str,
     opt: &Opt,
@@ -261,14 +280,49 @@ pub fn cut_str<W: Write>(
         bounds_as_ranges.drain(..1);
     }
 
+    if opt.only_delimited && bounds_as_ranges.len() == 1 {
+        // If there's only 1 field it means that there were no delimiters
+        // and when used alogside `only_delimited` we must skip the line
+        return Ok(());
+    }
+
+    if opt.json {
+        stdout.write_all(b"[")?;
+    }
+
+    let _bounds: UserBoundsList;
+    let mut bounds = &opt.bounds;
+
+    if opt.bounds_type == BoundsType::Characters && opt.replace_delimiter.is_some() {
+        // Unpack bounds such as 1:3 or 2: into single character bounds
+        // such as 1:1,2:2,3:3 etc...
+        // We need it to be able to insert a replace character between every field.
+        // It can cost quite a bit and is risky because it may end up creating a
+        // char vector of the whole input (then again -c with -r is quite the
+        // rare usage).
+
+        // Start by checking if we actually need to rewrite the bounds
+        if bounds.0.iter().any(|b| {
+            matches!(
+                b,
+                BoundOrFiller::Bound(UserBounds {
+                    l: x,
+                    r: y
+                }) if x != y || x == &Side::Continue
+            )
+        }) {
+            // Yep, there at least a range bound. Let's do it
+            _bounds = bounds.unpack(bounds_as_ranges.len());
+            bounds = &_bounds;
+        }
+    }
+
     match bounds_as_ranges.len() {
-        1 if opt.only_delimited => (),
-        1 => {
-            stdout.write_all(line.as_bytes())?;
-            stdout.write_all(eol)?;
+        1 if bounds.0.len() == 1 => {
+            write_maybe_as_json!(stdout, line, opt.json);
         }
         _ => {
-            opt.bounds
+            bounds
                 .0
                 .iter()
                 .enumerate()
@@ -281,25 +335,35 @@ pub fn cut_str<W: Write>(
                         BoundOrFiller::Bound(b) => b,
                     };
 
-                    let r_array = [bounds_to_std_range(bounds_as_ranges.len(), b)?];
-                    let mut r_iter = r_array.iter();
-                    let _complements;
-                    let mut n_ranges = 1;
+                    let mut r_array = vec![bounds_to_std_range(bounds_as_ranges.len(), b)?];
 
                     if opt.complement {
-                        _complements = complement_std_range(bounds_as_ranges.len(), &r_array[0]);
-                        r_iter = _complements.iter();
-                        n_ranges = _complements.len();
+                        r_array = complement_std_range(bounds_as_ranges.len(), &r_array[0]);
                     }
 
+                    if opt.json {
+                        r_array = r_array
+                            .iter()
+                            .flat_map(|r| r.start..r.end)
+                            .map(|i| Range {
+                                start: i,
+                                end: i + 1,
+                            })
+                            .collect();
+                    }
+
+                    let r_iter = r_array.iter();
+                    let n_ranges = r_array.len();
+
                     for (idx_r, r) in r_iter.enumerate() {
                         let idx_start = bounds_as_ranges[r.start].start;
                         let idx_end = bounds_as_ranges[r.end - 1].end;
                         let output = &line[idx_start..idx_end];
 
-                        stdout.write_all(maybe_replace_delimiter(output, opt).as_bytes())?;
+                        let field_to_print = maybe_replace_delimiter(output, opt);
+                        write_maybe_as_json!(stdout, field_to_print, opt.json);
 
-                        if opt.join && !(i == opt.bounds.0.len() - 1 && idx_r == n_ranges - 1) {
+                        if opt.join && !(i == bounds.0.len() - 1 && idx_r == n_ranges - 1) {
                             stdout.write_all(
                                 opt.replace_delimiter
                                     .as_ref()
@@ -311,11 +375,15 @@ pub fn cut_str<W: Write>(
 
                     Ok(())
                 })?;
-
-            stdout.write_all(eol)?;
         }
     }
 
+    if opt.json {
+        stdout.write_all(b"]")?;
+    }
+
+    stdout.write_all(eol)?;
+
     Ok(())
 }
 
@@ -379,14 +447,13 @@ mod tests {
     fn test_complement_std_range() {
         // remember, it assumes that ranges are "legit" (not out of bounds)
 
-        // test empty string
-        assert_eq!(complement_std_range(0, &(0..0)), vec![]);
+        let empty_vec: Vec<Range<usize>> = vec![];
 
         // test 1-long string
-        assert_eq!(complement_std_range(1, &(0..1)), vec![]);
+        assert_eq!(complement_std_range(1, &(0..1)), empty_vec);
 
         // test ranges that reach left or right bounds
-        assert_eq!(complement_std_range(5, &(0..5)), vec![]);
+        assert_eq!(complement_std_range(5, &(0..5)), empty_vec);
         assert_eq!(complement_std_range(5, &(0..3)), vec![3..5]);
         assert_eq!(complement_std_range(5, &(3..5)), vec![0..3]);
 
@@ -394,7 +461,7 @@ mod tests {
         assert_eq!(complement_std_range(5, &(1..3)), vec![0..1, 3..5]);
 
         // test 2-long string
-        assert_eq!(complement_std_range(2, &(0..2)), vec![]);
+        assert_eq!(complement_std_range(2, &(0..2)), empty_vec);
         assert_eq!(complement_std_range(2, &(0..1)), vec![1..2]);
         assert_eq!(complement_std_range(2, &(1..2)), vec![0..1]);
     }
@@ -402,12 +469,13 @@ mod tests {
     #[test]
     fn test_build_ranges_vec() {
         let mut v_range: Vec<Range<usize>> = Vec::new();
+        let empty_vec: Vec<Range<usize>> = vec![];
 
         // non greedy
 
         v_range.clear();
         build_ranges_vec(&mut v_range, "", "-", false);
-        assert_eq!(v_range, vec![]);
+        assert_eq!(v_range, vec![] as Vec<Range<usize>>);
 
         v_range.clear();
         build_ranges_vec(&mut v_range, "a", "-", false);
@@ -453,7 +521,7 @@ mod tests {
 
         v_range.clear();
         build_ranges_vec(&mut v_range, "", "-", true);
-        assert_eq!(v_range, vec![]);
+        assert_eq!(v_range, empty_vec);
 
         v_range.clear();
         build_ranges_vec(&mut v_range, "a", "-", true);
@@ -680,6 +748,23 @@ mod tests {
         assert_eq!(output, "🤩\n".as_bytes());
     }
 
+    #[test]
+    fn cut_str_it_cut_characters_and_replace_the_delimiter() {
+        let mut opt = make_fields_opt();
+        let (mut output, mut buffer1, mut buffer2) = make_cut_str_buffers();
+        let eol = &[EOL::Newline as u8];
+
+        let line = "😁🤩😝😎";
+        opt.bounds = UserBoundsList::from_str("1,2,3:4").unwrap();
+        opt.bounds_type = BoundsType::Characters;
+        opt.delimiter = String::new();
+        opt.replace_delimiter = Some("-".to_owned());
+        opt.join = true; // implied when using BoundsType::Characters
+
+        cut_str(line, &opt, &mut output, &mut buffer1, &mut buffer2, eol).unwrap();
+        assert_eq!(&String::from_utf8_lossy(&output), "😁-🤩-😝-😎\n");
+    }
+
     #[test]
     fn cut_str_it_supports_zero_terminated_lines() {
         let mut opt = make_fields_opt();
@@ -900,4 +985,90 @@ mod tests {
         cut_str(line, &opt, &mut output, &mut buffer1, &mut buffer2, eol).unwrap();
         assert_eq!(output, b"abc\n".as_slice());
     }
+
+    #[test]
+    fn cut_str_it_produce_json_output() {
+        let mut opt = make_fields_opt();
+        opt.json = true;
+        opt.replace_delimiter = Some(",".to_owned());
+        let (mut output, mut buffer1, mut buffer2) = make_cut_str_buffers();
+        let eol = &[EOL::Newline as u8];
+
+        let line = "a-b-c";
+        opt.bounds = UserBoundsList::from_str("1,3").unwrap();
+        opt.join = true;
+
+        cut_str(line, &opt, &mut output, &mut buffer1, &mut buffer2, eol).unwrap();
+        assert_eq!(
+            output,
+            br#"["a","c"]
+"#
+            .as_slice()
+        );
+    }
+
+    #[test]
+    fn cut_str_json_with_single_field_is_still_an_array() {
+        let mut opt = make_fields_opt();
+        opt.json = true;
+        opt.replace_delimiter = Some(",".to_owned());
+        let (mut output, mut buffer1, mut buffer2) = make_cut_str_buffers();
+        let eol = &[EOL::Newline as u8];
+
+        let line = "a-b-c";
+        opt.bounds = UserBoundsList::from_str("1").unwrap();
+        opt.join = true;
+
+        cut_str(line, &opt, &mut output, &mut buffer1, &mut buffer2, eol).unwrap();
+        assert_eq!(
+            output,
+            br#"["a"]
+"#
+            .as_slice()
+        );
+    }
+
+    #[test]
+    fn cut_str_complement_works_with_json() {
+        let mut opt = make_fields_opt();
+        opt.json = true;
+        opt.replace_delimiter = Some(",".to_owned());
+        opt.complement = true;
+        let (mut output, mut buffer1, mut buffer2) = make_cut_str_buffers();
+        let eol = &[EOL::Newline as u8];
+
+        let line = "a-b-c";
+        opt.bounds = UserBoundsList::from_str("2,2:3,-1").unwrap();
+        opt.join = true;
+
+        cut_str(line, &opt, &mut output, &mut buffer1, &mut buffer2, eol).unwrap();
+        assert_eq!(
+            output,
+            br#"["a","c","a","a","b"]
+"#
+            .as_slice()
+        );
+    }
+
+    #[test]
+    fn cut_str_json_on_characters_works() {
+        let mut opt = make_fields_opt();
+        let (mut output, mut buffer1, mut buffer2) = make_cut_str_buffers();
+        let eol = &[EOL::Newline as u8];
+
+        let line = "😁🤩😝😎";
+        opt.bounds = UserBoundsList::from_str("1,2,3:4").unwrap();
+        opt.bounds_type = BoundsType::Characters;
+        opt.delimiter = String::new();
+        opt.join = true;
+        opt.json = true;
+        opt.replace_delimiter = Some(",".to_owned());
+
+        cut_str(line, &opt, &mut output, &mut buffer1, &mut buffer2, eol).unwrap();
+        assert_eq!(
+            &String::from_utf8_lossy(&output),
+            r#"["\uD83D\uDE01","\uD83E\uDD29","\uD83D\uDE1D","\uD83D\uDE0E"]
+"#
+        );
+    }
 }
