@@ -1,4 +1,4 @@
-use crate::bounds::{BoundsType, UserBoundsList};
+use crate::bounds::{BoundOrFiller, BoundsType, UserBounds, UserBoundsList};
 use anyhow::Result;
 use std::str::FromStr;
 
@@ -33,6 +33,7 @@ pub struct Opt {
     pub version: bool,
     pub complement: bool,
     pub join: bool,
+    pub json: bool,
     #[cfg(feature = "regex")]
     pub regex_bag: Option<RegexBag>,
     #[cfg(not(feature = "regex"))]
@@ -44,7 +45,7 @@ impl Default for Opt {
         Opt {
             delimiter: String::from("-"),
             eol: EOL::Newline,
-            bounds: UserBoundsList(Vec::new()),
+            bounds: UserBoundsList(vec![BoundOrFiller::Bound(UserBounds::default())]),
             bounds_type: BoundsType::Fields,
             only_delimited: false,
             greedy_delimiter: false,
@@ -54,6 +55,7 @@ impl Default for Opt {
             version: false,
             complement: false,
             join: false,
+            json: false,
             regex_bag: None,
         }
     }
